/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : Extension to pthread library.
 ==============================================================================================================================================
 */
#ifndef PTHREAD_H_
#define PTHREAD_H_
#include <pthread.h>
#include <sys/types.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Initiating a thread attribute @attr with the default value.
 * @param attr          Pointer to thread attribute.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_attr_init ( pthread_attr_t* attr, const char* filename, int line );


/**
 * Set Detach state attribute.
 * @param attr          Pointer to the detach state attribute.
 * @param detachstate   It controls whether the thread is created in a detached state.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_attr_setdetachstate ( pthread_attr_t* attr, int detachstate, const char* filename, int line );


/**
 * Destroy a condition variable.
 * @param cond          Pointer to the condition variable.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_cond_destroy ( pthread_cond_t* cond, const char* filename, int line );


/**
 * Initialise a condition variable.
 * @param cond          Pointer to the condition variable.
 * @param attr          Pointer to the condition attribute.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_cond_init ( pthread_cond_t* cond, const pthread_condattr_t* attr, const char* filename, int line );


/**
 * Signal a condition.
 * @param cond          Pointer to the condition variable.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_cond_signal ( pthread_cond_t* cond, const char* filename, int line );


///**
// * Wait on a condition.
// * @param cond          Pointer to the condition variable.
// * @param mutex         Pointer to the mutual exclusion variable.
// * @param abstime       Absolute time.
// * @return Returns 0 on success; returns -1 on failure.
// */
//int Pthread_cond_timedwait ( pthread_cond_t* cond, pthread_mutex_t* mutex, const struct timespec* abstime );


/**
 * Wait on a condition.
 * @param cond          Pointer to the condition variable.
 * @param mutex         Pointer to the mutual exclusion variable.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_cond_wait ( pthread_cond_t* cond, pthread_mutex_t* mutex, const char* filename, int line );


/**
 * @param thread        Pointer to the thread descriptor.
 * @param attr          Pointer to the thread attribute.
 * @param start_routine Routine of thread.
 * @param arg           Arguments bing passed to @start_routine.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_create ( pthread_t* thread, const pthread_attr_t* attr, void*(*start_routine)(void*), void* arg,
    const char* filename, int line );


///**
// * Detach a thread.
// * @param thread        Thread descriptor.
// * @return Returns 0 on success; returns -1 on failure.
// */
//int Pthread_detach ( pthread_t thread );
//
//
///**
// * Wait for thread termination.
// * @param thread        Thread descriptor.
// * @param value_ptr     The value passed to pthread_exit() by the terminating thread is made by @value_ptr.
// * @return Returns 0 on success; returns -1 on failure.
// */
//int Pthread_join ( pthread_t thread, void** value_ptr );


/**
 * Destroy a mutual exclusion.
 * @param mutex         Pointer to the mutual exclusion descriptor.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_mutex_destroy ( pthread_mutex_t* mutex, const char* filename, int line );


/**
 * Initialise a mutex.
 * @param mutex         Pointer to the mutual exclusion descriptor.
 * @param attr          Pointer to the mutual exclusion attribute.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_mutex_init ( pthread_mutex_t* mutex, const pthread_mutexattr_t* attr, const char* filename, int line );


/**
 * Lock a mutual exclusion.
 * @param mutex         Pointer to the mutual exclusion descriptor.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_mutex_lock ( pthread_mutex_t* mutex, const char* filename, int line );


/**
 * Unlock a mutual exclusion.
 * @param mutex         Pointer to the mutual exclusion descriptor.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_mutex_unlock ( pthread_mutex_t* mutex, const char* filename, int line );


/**
 * Destroy a read-write lock object.
 * @param rwlock        Pointer to the read-write lock object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlock_destroy ( pthread_rwlock_t* rwlock, const char* filename, int line );


/**
 * Initialise a read-write lock object.
 * @param rwlock        Pointer to the read-write lock object.
 * @param attr          Read-write lock attribute.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlock_init ( pthread_rwlock_t* rwlock, const pthread_rwlockattr_t* attr, const char* filename, int line );


/**
 * Lock a read-write lock object for reading.
 * @param rwlock        Pointer to the read-write lock object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlock_rdlock ( pthread_rwlock_t* rwlock, const char* filename, int line );


/**
 * Unlock a read-write lock object.
 * @param rwlock        Pointer to the read-write lock object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlock_unlock ( pthread_rwlock_t* rwlock, const char* filename, int line );


/**
 * Lock a read-write lock object for writing.
 * @param rwlock        Pointer to the read-write lock object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlock_wrlock ( pthread_rwlock_t* rwlock, const char* filename, int line );


/**
 * Destroy an read-write lock attribute object.
 * @param attr          Pointer to the read-write attribute object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlockattr_destroy ( pthread_rwlockattr_t* attr, const char* filename, int line );


/**
 * Initialise an read-write lock attribute object.
 * @param attr          Pointer to the read-write attribute object.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlockattr_init ( pthread_rwlockattr_t* attr, const char* filename, int line );


/**
 * Set process-shared attribute of an read-write lock attribute object.
 * @param attr          Pointer to the read-write attribute object.
 * @param pshared       Process-shared attribute.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Pthread_rwlockattr_setpshared ( pthread_rwlockattr_t *attr, int pshared, const char* filename, int line );


/**
 * Set the read-write lock kind of the thread read-write lock attribute object.
 * @param attr          Pointer to the read-write attribute object.
 * @param pref          Preference or attribute.
 * @return Returns 0 on success; returns -1 on failure.
 * @see http://dev.linuxfoundation.org/snapshots/booksets/LSB-Core-generic/LSB-Core-generic/libpthread-pthread-rwlockattr-getkind-np-1.html
 */
int Pthread_rwlockattr_setkind_np ( pthread_rwlockattr_t* attr, int pref, const char* filename, int line );




#endif /* PTHREAD_H_ */
